
// Filename: top.sv

//----------------------------------------------------------------------
//  Copyright (c) 2010 by Doulos.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//----------------------------------------------------------------------

`include "ovm_macros.svh"

package my_pkg;

  `ifdef INCA 
    `include "ovm.svh"
  `else
    import ovm_pkg::*;
  `endif

  class my_tx extends ovm_sequence_item;
 
    // Multiplexed streams of 16-bit samples 
    rand logic        [3:0]  tag;
    rand logic signed [15:0] data;
    
    function new (string name = "");
      super.new(name);
    endfunction: new
  
    `ovm_object_utils_begin(my_tx)
      `ovm_field_int(tag,  OVM_ALL_ON + OVM_DEC)
      `ovm_field_int(data, OVM_ALL_ON + OVM_DEC)
    `ovm_object_utils_end
  
  endclass: my_tx
  
  
  class my_sequencer extends ovm_sequencer #(my_tx);

    `ovm_sequencer_utils(my_sequencer)

    function new (string name = "my_sequencer", ovm_component parent);
      super.new(name, parent);
      `ovm_update_sequence_lib_and_item(my_tx)
    endfunction : new

  endclass : my_sequencer

 
  import "DPI-C" function void initialize_fft(input int n_points);
  import "DPI-C" function void transfer_sample(input logic [3:0] tag, logic signed [15:0] data);
    
  class my_driver extends ovm_driver #(my_tx);

    `ovm_component_utils(my_driver)
  
    function new(string name, ovm_component parent);
      super.new(name, parent);
    endfunction: new
    
    virtual function void start_of_simulation;
      initialize_fft(32);
    endfunction
   
    virtual task run;

      forever
      begin
        my_tx tx;
        #10
        seq_item_port.get_next_item(tx);
        
        //ovm_report_info("", $psprintf("OVM TB sample tag = %2d, data = %6d}",
        //                                tx.tag, tx.data));
        
        transfer_sample(tx.tag, tx.data); // DPI call
        
        seq_item_port.item_done();
      end
    endtask: run
    
  endclass: my_driver


  class my_env extends ovm_env;

    `ovm_component_utils(my_env)
    
    my_sequencer                          m_sequencer;
    my_driver                             m_driver;
  
    function new(string name = "my_env", ovm_component parent = null);
      super.new(name, parent);
    endfunction: new
    
    virtual function void build;
      super.build();
      m_driver    = my_driver   ::type_id::create("m_driver"   , this);
      m_sequencer = my_sequencer::type_id::create("m_sequencer", this);
    endfunction: build
    
    virtual function void connect;
      m_driver.seq_item_port.connect(m_sequencer.seq_item_export);
    endfunction: connect

  endclass: my_env
endpackage: my_pkg
  

module top;

`ifdef INCA 
  `include "ovm.svh"
`else
  import ovm_pkg::*;
`endif

import my_pkg::*;
  
class my_test extends ovm_test;
  `ovm_component_utils(my_test)
    
  function new(string name, ovm_component parent);
    super.new(name,parent);
  endfunction: new

  my_env m_env;   
 
  virtual function void build;
    super.build();
    set_config_int("m_env.m_sequencer", "count", 32*16);
    m_env = my_env::type_id::create("m_env", this);
    //set_global_timeout(1us);
  endfunction: build
  
endclass: my_test

initial
  run_test("my_test");

endmodule

`ifdef NCSC
// Needed by Cadence only

module scmod ()
// The foreign attribute string value must be a SystemC value.
(* integer foreign = "SystemC"; *);
// Verilog port names must match port names exactly as they appear in the
// sc_module class in SystemC; they must also match in order, mode, and type.
endmodule

`endif
