
// Filename: scwrap.cpp

//----------------------------------------------------------------------
//  Copyright (c) 2010 by Doulos.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//----------------------------------------------------------------------

#include "scwrap.h"

#ifdef CADENCE
  NCSC_MODULE_EXPORT(scwrap);
#endif

#ifdef MENTOR
  SC_MODULE_EXPORT(scwrap);
#endif

#ifdef CADENCE
  // For MENTOR this is in "sc_dpiheader.h"
  extern "C"
  void tx_done(int id, char cmd, int addr, int data);
#endif

scwrap::scwrap(sc_module_name n)
: sc_module(n)
{
  m_scmod = new scmod("m_scmod");

  SC_THREAD(thread1);
    sensitive << dpi_event1;
  SC_THREAD(thread2);
    sensitive << dpi_event2;
  SC_THREAD(thread3);
    sensitive << dpi_event3;
  SC_THREAD(thread4);
    sensitive << dpi_event4;

  id = ++count;
  instance[id] = this;
  cout << "Called scwrap::scwrap(), id = " << id << ", path = " << name() << endl;

  #ifdef MENTOR
    SC_DPI_REGISTER_CPP_FUNCTION(entry);
    SC_DPI_REGISTER_CPP_FUNCTION(get_path);
  #endif
}

std::vector<scwrap*> scwrap::instance(0);  // Synopsys forbids definition of static variable in header file

int scwrap::count = 0;

static svScope  calling_scope;

#define THREAD_POOL(N) \
void scwrap::thread##N()\
{\
  for(;;)\
  {\
    wait(); \
    scwrap::instance[id]->m_scmod->execute(send_tx##N);\
    svSetScope(calling_scope);\
    tx_done(id, send_tx##N->cmd, send_tx##N->addr, send_tx##N->data);\
    delete send_tx##N;\
  }\
}

THREAD_POOL(1)
THREAD_POOL(2)
THREAD_POOL(3)
THREAD_POOL(4)

#ifndef CDN_OR_MEN
extern "C"
#endif
void entry(int id, char cmd, int addr, int* data)
{
  static int thread_n = 1;
  bus_t* tx = new bus_t;
  tx->cmd = cmd;
  tx->addr = addr;
  tx->data = *data;
  switch(thread_n)
  {
    case 1: scwrap::instance[id]->dpi_event1.notify();
            scwrap::instance[id]->send_tx1 = tx;
            break;
    case 2: scwrap::instance[id]->dpi_event2.notify();
            scwrap::instance[id]->send_tx2 = tx;
            break;
    case 3: scwrap::instance[id]->dpi_event3.notify();
            scwrap::instance[id]->send_tx3 = tx;
            break;
    case 4: scwrap::instance[id]->dpi_event4.notify();
            scwrap::instance[id]->send_tx4 = tx;
            break;
  }
  if (++thread_n > 4) thread_n = 1;
  //cout << "id = " << id << ", dpi_event.notify()" << endl;
  calling_scope = svGetScope();
}

#ifndef CDN_OR_MEN
extern "C"
#endif
const char* get_path(int id)
{
  return scwrap::instance[id]->name();
}

#ifdef CADENCE
  NCSC_REGISTER_DPI(entry)
  NCSC_REGISTER_DPI(get_path)
#endif
