
// Filename: top.sv

//----------------------------------------------------------------------
//  Copyright (c) 2010 by Doulos.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//----------------------------------------------------------------------

`include "vmm.sv"

class my_tx extends vmm_data;

  // Multiplexed streams of 16-bit samples 
  rand logic        [3:0]  tag;
  rand logic signed [15:0] data;
  
  `vmm_data_member_begin(my_tx)
    `vmm_data_member_scalar(tag,  DO_ALL)
    `vmm_data_member_scalar(data, DO_ALL)
  `vmm_data_member_end(my_tx)

endclass: my_tx


import "DPI-C" function void initialize_fft(input int n_points);
import "DPI-C" function void transfer_sample(input logic [3:0] tag, logic signed [15:0] data);
  
class my_driver extends vmm_xactor;

  function new (string name, string instance);
    super.new(name, instance);
    initialize_fft(32);
  endfunction: new
  
  virtual protected task main;   // "start" for vmm_xactor. Called from start_xactor
    super.main();

    forever
    begin
      my_tx tx;
      #10
      tx = new;
      tx.randomize();
      
      //`vmm_note(log, $psprintf("VMM TB sample tag = %2d, data = %6d}",
      //                                tx.tag, tx.data));
      
      transfer_sample(tx.tag, tx.data); // DPI call
  
    end
  endtask
  
endclass: my_driver


class my_env extends vmm_env;

  my_driver m_driver;

  virtual function void build;
    super.build();
    m_driver  = new( "my_driver", "m_driver");
  endfunction: build
  
  virtual task start;                    // Standard phase method to start sim
  
    super.start();
    m_driver.start_xactor();
    
  endtask: start
  
  virtual task wait_for_end;             // Standard phase method 
  
    super.wait_for_end();
    `vmm_note(log, "Called my_env::wait_for_end");
    #(32*16*10);                                // Determines when sim ends by blocking
    
  endtask: wait_for_end

endclass: my_env



program test;                            // Should be a program, not a module

  my_env env = new;
  
  initial
  begin
    env.run();
  end
    
endprogram: test

