
// Filename: top.sv

//----------------------------------------------------------------------
//  Copyright (c) 2010 by Doulos.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//----------------------------------------------------------------------

timeunit 1ns;
timeprecision 1ns;

`include "vmm.sv"

class my_tx extends vmm_data;

  typedef enum byte { READ, WRITE } cmd_t;  

  rand cmd_t cmd;
  rand int   addr;
  rand int   data;

  constraint c_addr { addr >= 0; addr < 256; }
  constraint c_data { data >= 0; data < 256; }

  `vmm_data_member_begin(my_tx)
    `vmm_data_member_enum(cmd,  DO_ALL)
    `vmm_data_member_scalar(addr, DO_ALL)
    `vmm_data_member_scalar(data, DO_ALL)
  `vmm_data_member_end(my_tx)

endclass: my_tx

import "DPI-C"  function void   entry(input int id, input byte cmd, input int addr, inout int data);
import "DPI-C"  function string get_path(input int id);
export "DPI-C"  function tx_done;

int path_to_id[string];
event done_ev[5];
    
function void tx_done(input int id, input byte cmd, input int addr, input int data);
  $display("             vvvvv Called tx_done at %4d, id = %1d, cmd = %1d, addr = %3d, data = %3d",
           $time, id, cmd, addr, data);
  ->done_ev[id];
endfunction
  
class my_driver extends vmm_xactor;

  function new (string name, string instance);
    super.new(name, instance);

    path_to_id[get_path(1)] = 1;
    path_to_id[get_path(2)] = 2;
    
  endfunction: new

  virtual protected task main;   // "start" for vmm_xactor. Called from start_xactor
    super.main();

    forever
    begin
      my_tx tx;
      int id;
      #10
      tx = new;
      tx.randomize();
      
      `vmm_note(log, $psprintf("VMM TB driving cmd = %1d, addr = %3d, data = %3d}",
                               tx.cmd, tx.addr, tx.data));

      if ($urandom_range(0,1))
        id = path_to_id["top_m_hier_m_scwrap_1"];
      else
        id = path_to_id["top_m_hier_m_scwrap_2"];

      entry(id, tx.cmd, tx.addr, tx.data); 
      
      // Wait until tx done to avoid pipelining
      //@(done_ev[id]);
      //`vmm_note(log, "VMM TB tx done");
      
    end
  endtask
  
endclass: my_driver

    
class my_env extends vmm_env;

  my_driver m_driver;

  virtual function void build;
    super.build();
    m_driver  = new( "my_driver", "m_driver");
  endfunction: build
  
  virtual task start;                    // Standard phase method to start sim
    super.start();
    m_driver.start_xactor();
  endtask: start
  
  virtual task wait_for_end;             // Standard phase method 
    super.wait_for_end();
    `vmm_note(log, "Called my_env::wait_for_end");
    #(20*10);                                // Determines when sim ends by blocking
  endtask: wait_for_end

endclass: my_env


module hier;

scwrap m_scwrap_1();
scwrap m_scwrap_2();

endmodule

  
module top;

hier m_hier();

endmodule


program test;

  my_env env = new;
  
  initial
  begin
    env.run();
  end
    
endprogram: test
